function [w,wraw]=WvSampleMN(y,D,z,x,pzt,dpzt,dx,Exclude);
%dpzt is assumed to be n by (M*d) dimensional matrix. Each row contains a
%vectorized version of the derivative of the choice probability wrt to the 
%parameters organized by the ordering of the choice model.
%ie. the first M elements contain the weighted d derivatives of the first
%choice, the second M elements contain the d derivatives of the second
%choice and so on. D and pzt are n by M matrices of choices and choice
%probability for each of the M categories. 

tol=10^-5;

n=size(y,1);
dim=size(pzt,2); %dim=caligraphic M; see paper
d=size(dpzt,2)/dim;
corr=size(n,1);

%compute P(zt,theta)=diag(p(zt,theta))
Pzt=zeros(n,dim^2);
ind=find((vec(eye(dim))'==1));
Pzt(:,ind)=pzt;
InvPzt=zeros(n,dim^2);
hInv=zeros(n,dim^2);
for i=1:n
    if isempty(Exclude) || size(find(dx(i)*ones(length(Exclude),1)-Exclude),1)==size(Exclude,1)
        InvPzt(i,ind)=1./pzt(i,:);
        hInv(i,:)=InvPzt(i,:)+(1/(1-pzt(i,:)*ones(dim,1)))*vec(ones(dim,1)*ones(1,dim))';
        lbar(i,:)=vec(reshape(hInv(i,:),dim,dim)*reshape(dpzt(i,:)',dim,d))';
        lbari=reshape(lbar(i,:),dim,d);
        b(i,:)=(lbari'*((D(i,:)-pzt(i,:)))')';
        ClS(i,:)=vec(lbari'*(diag(pzt(i,:)')-pzt(i,:)'*pzt(i,:))*lbari);
    end        
end
M=(y(:,1)*ones(1,n)>ones(n,1)*y(:,1)');
InnerSum=M'*b;
cSamp=M'*ClS/n;
w=zeros(n,dim);
wraw=zeros(n,dim);
for i=1:n;
    if isempty(Exclude) || size(find(dx(i)*ones(length(Exclude),1)-Exclude),1)==size(Exclude,1)       
        cSampi=reshape(cSamp(i,:),d,d);
        cSampi=(cSampi+cSampi')/2; %enforce symmetry
        detcSamp(i,:)=det(cSampi);        
        [Ve,De]=eig(cSampi);
        [dDe,indDe]=sort(diag(De),'descend');
        Ve=Ve(:,indDe);            
        rcS=max(find(dDe>tol));
        if isempty(rcS)
            rcS=0;
            rcSi(i,:)=0;
        else
            rcSi(i,:)=rcS;
        end        
%        if rcS<size(cSampi,1)
%            c=zeros(d,d);                 
%        else
            c=diag(1./dDe(1:rcS,1)); %bring order 
            c=[[c,zeros(rcS,d-rcS)];zeros(d-rcS,d)];                
            c=Ve*c*Ve'; %generalized Inverse
%        end
        correct(i,:)=(reshape(dpzt(i,:),dim,d)*c*InnerSum(i,:)'/n)';
        hInvi=reshape(hInv(i,:),dim,dim);
        w(i,:)=((D(i,:)-pzt(i,:)-correct(i,:))*sqrtm(hInvi))';
        wraw(i,:)=D(i,:)-pzt(i,:);
    end
end
return

